var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var ApprovalTrigger_exports = {};
__export(ApprovalTrigger_exports, {
  default: () => ApprovalTrigger
});
module.exports = __toCommonJS(ApprovalTrigger_exports);
var import_lodash = require("lodash");
var import_sequelize = require("sequelize");
var import_database = require("@nocobase/database");
var import_data_source_manager = require("@nocobase/data-source-manager");
var import_plugin_workflow = require("@nocobase/plugin-workflow");
var import_constants = require("./constants");
const ExecutionStatusMap = {
  [import_plugin_workflow.EXECUTION_STATUS.RESOLVED]: import_constants.APPROVAL_STATUS.APPROVED,
  [import_plugin_workflow.EXECUTION_STATUS.REJECTED]: import_constants.APPROVAL_STATUS.REJECTED,
  [import_plugin_workflow.EXECUTION_STATUS.CANCELED]: import_constants.APPROVAL_STATUS.DRAFT,
  [import_plugin_workflow.EXECUTION_STATUS.RETRY_NEEDED]: import_constants.APPROVAL_STATUS.RETURNED
  // FAILED: -1,
  // ERROR: -2,
  // ABORTED: -3,
};
const ApprovalJobStatusMap = {
  [import_constants.APPROVAL_ACTION_STATUS.APPROVED]: import_constants.APPROVAL_STATUS.APPROVED,
  [import_constants.APPROVAL_ACTION_STATUS.REJECTED]: import_constants.APPROVAL_STATUS.REJECTED,
  [import_constants.APPROVAL_ACTION_STATUS.RETURNED]: import_constants.APPROVAL_STATUS.RETURNED
};
class ApprovalTrigger extends import_plugin_workflow.Trigger {
  static TYPE = "approval";
  sync = false;
  triggerHandler = async (approval, { transaction }) => {
    const workflow = await approval.getWorkflow({
      where: {
        id: approval.get("workflowId"),
        type: ApprovalTrigger.TYPE,
        enabled: true,
        "config.collection": approval.collectionName
      },
      transaction
    });
    if (!workflow || !approval.changed("status") || approval.status !== import_constants.APPROVAL_STATUS.SUBMITTED) {
      return;
    }
    const [dataSourceName, collectionName] = (0, import_data_source_manager.parseCollectionName)(approval.collectionName);
    const { repository } = this.workflow.app.dataSourceManager.dataSources.get(dataSourceName).collectionManager.getCollection(collectionName);
    const data = await repository.findOne({
      filterByTk: approval.get("dataKey"),
      appends: workflow.config.appends,
      transaction: this.workflow.useDataSourceTransaction(dataSourceName, transaction)
    });
    this.workflow.trigger(
      workflow,
      {
        data: (0, import_plugin_workflow.toJSON)(data),
        approvalId: approval.id,
        applicantRoleName: approval.applicantRoleName
      },
      { transaction }
    );
  };
  onExecutionCreate = async (execution, { transaction }) => {
    const workflow = await execution.getWorkflow({ transaction });
    if (workflow.type !== ApprovalTrigger.TYPE) {
      return;
    }
    const { approvalId, data } = execution.context;
    const approvalExecution = await this.workflow.db.getRepository("approvalExecutions").create({
      values: {
        approvalId,
        executionId: execution.id,
        status: execution.status,
        snapshot: data
      },
      transaction
    });
    await this.workflow.db.getRepository("approvals").update({
      filterByTk: approvalId,
      values: { latestExecutionId: approvalExecution.id },
      transaction
    });
  };
  onExecutionUpdate = async (execution, { transaction }) => {
    if (!execution.workflow) {
      execution.workflow = await execution.getWorkflow({ transaction });
    }
    if (!execution.status || execution.workflow.type !== ApprovalTrigger.TYPE) {
      return;
    }
    const approvalExecution = await this.workflow.db.getRepository("approvalExecutions").findOne({
      filter: {
        executionId: execution.id,
        approvalId: execution.context.approvalId
      },
      appends: ["approval"],
      transaction
    });
    const { approval } = approvalExecution;
    if (![import_constants.APPROVAL_STATUS.SUBMITTED, import_constants.APPROVAL_STATUS.PROCESSING].includes(approval.status)) {
      return;
    }
    let status = import_constants.APPROVAL_STATUS.APPROVED;
    if (execution.status === import_plugin_workflow.EXECUTION_STATUS.RESOLVED) {
      const [approvalNodeJob] = await execution.getJobs({
        where: {
          status: {
            [import_sequelize.Op.ne]: import_plugin_workflow.JOB_STATUS.PENDING
          }
        },
        include: [
          {
            association: "node",
            where: {
              type: "approval"
            },
            required: true
          }
        ],
        order: [["updatedAt", "DESC"]],
        limit: 1,
        transaction
      });
      if (approvalNodeJob && ApprovalJobStatusMap[approvalNodeJob.result] != null) {
        status = ApprovalJobStatusMap[approvalNodeJob.result];
      }
    } else {
      if (ExecutionStatusMap[execution.status] != null) {
        status = ExecutionStatusMap[execution.status];
      }
    }
    await approval.update({ status }, { transaction });
    await approvalExecution.update({ status: execution.status }, { transaction });
  };
  middleware = async (context, next) => {
    const {
      resourceName,
      actionName,
      params: { triggerWorkflows }
    } = context.action;
    if (resourceName === "workflows" && actionName === "trigger") {
      return this.workflowTriggerAction(context, next);
    }
    await next();
    if (!triggerWorkflows || !["create", "update"].includes(actionName)) {
      return;
    }
    this.collectionTriggerAction(context);
  };
  constructor(workflow) {
    super(workflow);
    const { db } = workflow.app;
    db.on("approvals.afterSave", this.triggerHandler);
    db.on("executions.afterCreate", this.onExecutionCreate);
    db.on("executions.afterUpdate", this.onExecutionUpdate);
    workflow.app.use(this.middleware, { after: "dataSource" });
  }
  async workflowTriggerAction(context, next) {
    const { triggerWorkflows, values } = context.action.params;
    if (!triggerWorkflows) {
      return context.throw(400);
    }
    const triggers = triggerWorkflows.split(",").map((trigger) => trigger.split("!"));
    const workflowRepo = this.workflow.db.getRepository("workflows");
    const workflows = await workflowRepo.find({
      filter: {
        type: ApprovalTrigger.TYPE,
        key: triggers.map((trigger) => trigger[0]),
        current: true,
        enabled: true
      }
    });
    context.status = 202;
    await next();
    workflows.forEach(async (workflow) => {
      const trigger = triggers.find((trigger2) => trigger2[0] === workflow.key);
      const [dataSourceName, collectionName] = (0, import_data_source_manager.parseCollectionName)(workflow.config.collection);
      const collecton = this.workflow.app.dataSourceManager.dataSources.get(dataSourceName).collectionManager.getCollection(collectionName);
      const data = await collecton.repository.create({
        values: {
          ...trigger[1] ? (0, import_lodash.get)(values, trigger[1]) : values
          // createdBy: currentUser.id,
          // updatedById: currentUser.id,
        },
        context
      });
      const approvalRepo = this.workflow.db.getRepository("approvals");
      await approvalRepo.create({
        values: {
          collectionName: workflow.config.collection,
          data: (0, import_plugin_workflow.toJSON)(data),
          dataKey: data.get(collecton.filterTargetKey),
          status: import_constants.APPROVAL_STATUS.SUBMITTED,
          // createdBy: currentUser.id,
          // updatedById: currentUser.id,
          workflowId: workflow.id,
          workflowKey: workflow.key
        },
        context
      });
    });
  }
  async collectionTriggerAction(context) {
    const { triggerWorkflows = "" } = context.action.params;
    const dataSourceHeader = context.get("x-data-source") || "main";
    const approvalRepo = this.workflow.db.getRepository("approvals");
    const triggers = triggerWorkflows.split(",").map((trigger) => trigger.split("!"));
    const triggersKeysMap = new Map(triggers);
    const workflows = Array.from(this.workflow.enabledCache.values()).filter(
      (item) => item.type === ApprovalTrigger.TYPE && triggersKeysMap.has(item.key)
    );
    for (const workflow of workflows) {
      const [dataSourceName, collectionName] = (0, import_data_source_manager.parseCollectionName)(workflow.config.collection);
      const trigger = triggers.find((trigger2) => trigger2[0] === workflow.key);
      const { body: data } = context;
      if (!data) {
        return;
      }
      if (dataSourceName !== dataSourceHeader) {
        continue;
      }
      (Array.isArray(data) ? data : [data]).forEach(async (row) => {
        let payload = row;
        if (trigger[1]) {
          const paths = trigger[1].split(".");
          for await (const field of paths) {
            if (payload.get(field)) {
              payload = payload.get(field);
            } else {
              const association = (0, import_database.modelAssociationByKey)(payload, field);
              payload = await payload[association.accessors.get]();
            }
          }
        }
        const collection = context.app.dataSourceManager.dataSources.get(dataSourceName).collectionManager.getCollection(collectionName);
        if (!collection || collection.model !== payload.constructor) {
          return;
        }
        await approvalRepo.create({
          values: {
            collectionName: workflow.config.collection,
            data: (0, import_plugin_workflow.toJSON)(payload),
            dataKey: payload.get(collection.filterTargetKey),
            status: import_constants.APPROVAL_STATUS.SUBMITTED,
            // createdBy: currentUser.id,
            // updatedBy: currentUser.id,
            workflowId: workflow.id,
            workflowKey: workflow.key,
            applicantRoleName: context.state.currentRole
          },
          context
        });
      });
    }
  }
  on(workflow) {
  }
  off(workflow) {
  }
  async duplicateConfig(workflow, { transaction }) {
    const { db } = this.workflow;
    const uiSchemaRepo = db.getRepository("uiSchemas");
    if (!workflow.config.applyForm) {
      return workflow.config;
    }
    const result = await uiSchemaRepo.duplicate(workflow.config.applyForm, {
      transaction
    });
    return {
      ...workflow.config,
      applyForm: result["x-uid"]
    };
  }
}
